LoggerConfig = {
	enable = true,
	maxRecords = 1000,
	maxTime = 0,
	displayAll = true,
	displaySelection = {},
	loaded = true,
	deaths = true,
	zoneChange = true,
};
LoggerLog = {};
LoggerItemColor = {};
LoggerSelectionTop = 0;
LoggerSelectionBottom = 0;
LoggerDisplay = {};

LoggerTypes = {
	[1] = {
		var = "loaded",
		color = { ["r"] = 1,  ["g"] = 1, ["b"] = 0 },
		onEvent = function()
			LoggerAddRecord(1);
		end,
		onRead = function()
			return LOGGER_UILOADED;
		end
	},
	[2] = {
		event = "PLAYER_DEAD",
		var = "deaths",
		color = { ["r"] = 1,  ["g"] = .2, ["b"] = .2 },
		onEvent = function()
			LoggerAddRecord(2);
		end,
		onRead = function()
			return LOGGER_PLAYERDIED;
		end
	},
	[3] = {
		event = "COMBAT_LOG_EVENT",
		var = "kills",
		color = { ["r"] = .8,  ["g"] = .8, ["b"] = .8 },
		onEvent = function(...)
			local timestamp, subevent, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName = CombatLogGetCurrentEventInfo();

			if ( subevent == "PARTY_KILL" ) then

				if ( sourceName and destName ) then
					if ( sourceName == GetUnitName("player") ) then
						sourceName = "p";
					end
					LoggerAddRecord(3, destName, sourceName);
				end

			end
			--[[
			local mob, slayer;
			_, _, mob = string.find(arg1, LOGGER_YOUHAVESLAIN);
			if (  mob ) then
				slayer = "p";	-- p means player
			else
				_, _, mob, slayer = string.find(arg1, LOGGER_SLAINBY);
			end

			if ( mob and slayer ) then
				LoggerAddRecord(3, mob, slayer );
			end
			]]
		end,
		onRead = function(n)
			local name;
			if ( LoggerLog[n][4] == "p" ) then
				name = LOGGER_YOU;
			else
				name = LoggerLog[n][4];
			end
			return string.format( LOGGER_MOBSLAIN, LoggerLog[n][3] or "(nil)", name or "(nil)" );
		end
	},
	[4] = {
		event = "CHAT_MSG_LOOT",
		var = "loot",
		color = { ["r"] = 0,  ["g"] = .7, ["b"] = 0 },
		onEvent = function(arg1)
			local item, num, color, looter;
			_, _, color, item, num = string.find(arg1, LOGGER_LOOT1);
			if ( not item ) then
				_, _, looter, color, item, num = string.find(arg1, LOGGER_LOOT2);
			else
				looter = "p";
			end

			if ( looter and item ) then

				if ( LoggerItemColor[color] ) then
					color = LoggerItemColor[color];
				else
					color = 1;
				end
				if ( not num or num == "" ) then
					LoggerAddRecord(4, item, looter, color);
				else
					LoggerAddRecord(4, item, looter, color, tonumber(num));
				end
			end
		end,
		onRead = function(n)
			local name, item, color;

			if ( LoggerLog[n][4] == "p" ) then
				name = LOGGER_YOU;
			else
				name = LoggerLog[n][4];
			end
			_, _, _, color = GetItemQualityColor(LoggerLog[n][5]);

			item = "|c"..color..LoggerLog[n][3].."|r";
			if ( LoggerLog[n][6] ) then
				item = item.."x"..LoggerLog[n][6];
			end
			return string.format( LOGGER_ITEMLOOTED, item, name );
		end
	},
	[5] = {
		event = "CHAT_MSG_COMBAT_FACTION_CHANGE",
		var = "faction",
		color = { ["r"] = .5,  ["g"] = .5, ["b"] = 1 },
		onEvent = function(arg1)
			local faction, value;
			_, _, faction, value = string.find(arg1, LOGGER_REPUTATION1);
			if ( not faction ) then
				_, _, faction, value = string.find(arg1, LOGGER_REPUTATION2);
				if ( value ) then
					value = -tonumber(value);
				end
			end

			if ( faction and value ) then
				LoggerAddRecord(5, tonumber(value), faction);
			end
		end,
		onRead = function(n)
			local name;
			if ( LoggerLog[n][3] < 0 ) then
				name = tostring(LoggerLog[n][3]);
			else
				name = "+"..LoggerLog[n][3];
			end
			return string.format( LOGGER_FACTION, name, LoggerLog[n][4] );
		end
	},
	[6] = {
		event = "ZONE_CHANGED_NEW_AREA",
		var = "zoneChange",
		color = { ["r"] = 1,  ["g"] = 1,  ["b"] = 0 },
		onEvent = function()
			LoggerAddRecord(6, GetZoneText());
		end,
		onRead = function(n)
			return string.format( LOGGER_NEWZONE, LoggerLog[n][3] );
		end
	},
	[7] = {
		color = { ["r"] = 1,  ["g"] = 1,  ["b"] = 1 },
		onEvent = function(arg)
			LoggerAddRecord(7, arg);
		end,
		onRead = function(n)
			return LoggerLog[n][3];
		end
	},
	[8] = {
		event = "CHAT_MSG_WHISPER",
		var = "tellsReceived",
		color = { ["r"] = 1,  ["g"] = .5,  ["b"] = 1 },
		onEvent = function(arg1, arg2)
			LoggerAddRecord(8, arg1, arg2);
		end,
		onRead = function(n)
			return string.format( LOGGER_TELL_RECEIVED, LoggerLog[n][4], LoggerLog[n][3] );
		end
	},
	[9] = {
		event = "CHAT_MSG_WHISPER_INFORM",
		var = "tellsSent",
		color = { ["r"] = 1,  ["g"] = .5,  ["b"] = 1 },
		onEvent = function(arg1, arg2)
			LoggerAddRecord(9, arg1, arg2);
		end,
		onRead = function(n)
			return string.format( LOGGER_TELL_SENT, LoggerLog[n][4], LoggerLog[n][3] );
		end
	},
	[10] = {
		event = "CHAT_MSG_COMBAT_XP_GAIN",
		var = "exp",
		color = { ["r"] = .5,  ["g"] = .5, ["b"] = 1 },
		onEvent = function(arg1)
			local experience;
			_, _, experience = string.find(arg1, LOGGER_EXPERIENCE_GAIN);
			if ( experience ) then
				LoggerAddRecord(10, tonumber(experience));
			end
		end,
		onRead = function(n)
			return string.format( LOGGER_EXPERIENCE_GAIN_LOG, LoggerLog[n][3] );
		end
	},
	[11] = {
		event = "CHAT_MSG_SKILL",
		var = "skills",
		color = { ["r"] = .4,  ["g"] = .4, ["b"] = 1 },
		onEvent = function(arg1)
			local skill, n;
			_, _, skill, n = string.find(arg1, LOGGER_SKILL_GAIN);
			if ( skill ) then
				LoggerAddRecord(11, skill, tonumber(n));
			end
		end,
		onRead = function(n)
			return string.format( LOGGER_SKILL_GAIN_LOG, LoggerLog[n][3], LoggerLog[n][4] );
		end
	},
	[12] = {
		event = "CHAT_MSG_LOOT",
		var = "creations",
		color = { ["r"] = 0,  ["g"] = .7, ["b"] = 0 },
		onEvent = function(arg1)
			local item, num, color, looter;
			_, _, color, item, num = string.find(arg1, LOGGER_LOOT3);

			if ( item ) then

				if ( LoggerItemColor[color] ) then
					color = LoggerItemColor[color];
				else
					color = 1;
				end
				if ( not num or num == "" ) then
					LoggerAddRecord(12, item, color);
				else
					LoggerAddRecord(12, item, color, tonumber(num));
				end
			end
		end,
		onRead = function(n)
			local item, color;

			_, _, _, color = GetItemQualityColor(LoggerLog[n][4]);

			item = "|c"..color..LoggerLog[n][3].."|r";
			if ( LoggerLog[n][5] ) then
				item = item.."x"..LoggerLog[n][5];
			end
			return string.format( LOGGER_ITEMCREATED, item );
		end
	},
	[13] = {
		event = "PLAYER_LEVEL_UP",
		var = "levelup",
		color = { ["r"] = 1,  ["g"] = 1, ["b"] = 0 },
		onEvent = function(arg1, arg2, arg3)
			LoggerAddRecord(13, tonumber(arg1), tonumber(arg2), tonumber(arg3));
		end,
		onRead = function(n)
			return string.format( LOGGER_LEVELUP, LoggerLog[n][3], LoggerLog[n][4], LoggerLog[n][5] );
		end
	},
	[14] = {
		event = "CHAT_MSG_MONEY",
		var = "money",
		color = { ["r"] = 1,  ["g"] = 1, ["b"] = 0 },
		onEvent = function(arg1)
			local copper, silver, gold;
			_, _, copper = string.find(arg1, " (%d+) "..LOGGER_COPPER);
			_, _, silver = string.find(arg1, " (%d+) "..LOGGER_SILVER);
			_, _, gold = string.find(arg1, " (%d+) "..LOGGER_GOLD);

			if ( copper or silver or gold ) then
				LoggerAddRecord(14, tonumber(copper) or 0, tonumber(silver) or 0, tonumber(gold) or 0);
			end
		end,
		onRead = function(n)
			local copper, silver, gold = LoggerLog[n][3] or 0, LoggerLog[n][4] or 0, LoggerLog[n][5] or 0;
			local cs, ss, gs = "", "", "";
			if ( gold ~= 0 ) then
				gs = gold.." "..LOGGER_GOLD;
			end
			if ( silver ~= 0 ) then
				ss = silver.." "..LOGGER_SILVER;
				if ( gold ~= 0 ) then
					ss = ", "..ss;
				end
			end
			if ( copper ~= 0 ) then
				cs = copper.." "..LOGGER_COPPER;
				if ( gold ~= 0 or silver ~= 0 ) then
					cs = ", "..cs;
				end
			end

			return LOGGER_COIN_LOOT..gs..ss..cs;
		end
	},

};

LoggerEvents = {};

function LoggerFrame_OnLoad(self)
	self:RegisterEvent("VARIABLES_LOADED");

	local caps = string.upper(LOGGER_SLASHCMD);
	SlashCmdList[caps] = LoggerCmdLine;
	SLASH_LOGGER1 = "/"..LOGGER_SLASHCMD;

	-- get item color hex values
	local i, c;
	for i = 0, 6 do
		_, _, _, c = GetItemQualityColor(i);
		c = string.sub(c, 3);
		LoggerItemColor[c] = i;
	end
	
	-- make scroll bar arrow buttons only move one line
	LoggerSetScrollBarButtons();

	-- make an array with event strings pointing to the type numbers for fast indexing
	for i in pairs(LoggerTypes) do
		if ( LoggerTypes[i].event ) then
			if ( not LoggerEvents[LoggerTypes[i].event] ) then
				LoggerEvents[LoggerTypes[i].event] = {};
			end
			table.insert(LoggerEvents[LoggerTypes[i].event], i);
		end
	end

	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(LOGGER_LOGINMSG);
	end

end

function LoggerFrame_OnEvent(event, ...)

	if ( event == "VARIABLES_LOADED" ) then

		-- register events
		LoggerSetEvents();

		-- add UI loaded entry
		if ( LoggerConfig[LoggerTypes[1].var] ) then
			LoggerTypes[1].onEvent();
		end

		-- update total entries in UI
		LoggerFrameEntries:SetText(LOGGER_ENTRIES..#LoggerLog);

		-- set displayed entries
		LoggerSetDisplay();

		-- set scrollbar to bottom
		LoggerScrollToBottom = true;

		return
	end
	
	if ( LoggerEvents[event] ) then
		local i;
		for _, i in ipairs(LoggerEvents[event]) do
			if ( LoggerConfig[LoggerTypes[i].var] ) then
				LoggerTypes[i].onEvent(...);
			end
		end
	end
end

function LoggerSetEvents()
	local frame = LoggerFrame;

	local event, typeArr, i;

	-- unregister all events
	for event in pairs(LoggerEvents) do
		frame:UnregisterEvent(event);
	end

	-- if enabled, register events of events toggled on
	if ( LoggerConfig.enable ) then

		for event, typeArr in pairs(LoggerEvents) do
			for _, i in ipairs(typeArr) do
				if ( LoggerConfig[LoggerTypes[i].var] ) then
					frame:RegisterEvent(event);
					break;
				end
			end
		end
	end
end

function LoggerCmdLine(arg)
	_, _, arg1, arg2 = string.find(arg, "^%s*(%S+)%s+(.+)$");
	if (not arg1) then
		arg1 = arg;
	end
	arg1 = string.lower(arg1);

	if (arg1 == "debug") then

	elseif (arg1 == "comment") then
		if ( arg2 ) then
			LoggerAddRecord(7, arg2);
		end

	elseif (arg1 == "?" or arg1 == "help") then
		local i = 1;
		while (_G["LOGGER_HELP"..i]) do
			DEFAULT_CHAT_FRAME:AddMessage(_G["LOGGER_HELP"..i]);
			i = i + 1;
		end

	else
		DEFAULT_CHAT_FRAME:AddMessage(LOGGER_HELP0);
		LoggerFrame:Show();
	end

end

function LoggerAddRecord(type, ...)

	local dateStamp = time(date("*t"));        -- today's date + time in seconds (one big number)
	local t = { [1] = type, [2] = dateStamp };
	local arg, i;

	for i = 1, select("#", ...) do
		arg = select(i, ...);
		t[i + 2] = arg;
	end

	table.insert(LoggerLog, t);

	local recordsRemoved = 0;
	local entries = #LoggerLog;

	if ( entries == LoggerConfig.maxRecords + 1 ) then
		recordsRemoved = 1;
		table.remove(LoggerLog, 1);

	elseif ( entries > LoggerConfig.maxRecords ) then
		recordsRemoved = entries - LoggerConfig.maxRecords;
		local i;
		for i = 1, recordsRemoved do
			table.remove(LoggerLog, 1);
		end
	end

	-- remove too old records
	if ( LoggerConfig.maxTime > 0 ) then
		local n = 1;
		while ( dateStamp - LoggerLog[n][2] > LoggerConfig.maxTime ) do
			table.remove(LoggerLog, 1);
			recordsRemoved = recordsRemoved + 1;
		end
	end

	if ( recordsRemoved > 0 ) then
		-- adjust selection numbers to match new entry positions

		-- if top selection was deleted
		if ( LoggerSelectionTop <= recordsRemoved ) then
			LoggerSelectionTop = 1;		-- set new selection to first entry

		-- if top selection was not deleted
		elseif ( LoggerSelectionTop > recordsRemoved ) then
			LoggerSelectionTop = LoggerSelectionTop - recordsRemoved;	-- adjust selected entry to new position
		end

		-- if selected bottom entry is still line 2 or greater, then adjust position
		if ( LoggerSelectionBottom > (recordsRemoved + 1) ) then
			LoggerSelectionBottom = LoggerSelectionBottom - recordsRemoved;
		else
			LoggerSelectionBottom = 0;
			LoggerSelectionTop = 0;
		end
	end

	LoggerSetDisplayPending = true;

	if ( LoggerFrame:IsVisible() ) then

		-- is scroll bar at the bottom?
		local isMax = false;
		local _, max = LoggerScrollFrameScrollBar:GetMinMaxValues();
		if ( max == LoggerScrollFrameScrollBar:GetValue() ) then
			isMax = true;
		end

		LoggerFrame_Update();

		if ( isMax ) then
			-- if scroll bar was at the bottom, move it back there
			_, max = LoggerScrollFrameScrollBar:GetMinMaxValues();
			LoggerScrollFrameScrollBar:SetValue(max);
			LoggerFrame_Update();
		end
	end
end

function LoggerReadRecord(n)
	return LoggerLog[n][1], date("%D %T", LoggerLog[n][2]), LoggerTypes[LoggerLog[n][1]].onRead(n);
end

function LoggerSetDisplay()

	LoggerDisplay = {};
	if ( not LoggerConfig.displaySelection ) then
		LoggerConfig.displaySelection = {};
	end

	local i, v;

	for i, v in ipairs(LoggerLog) do
		if ( LoggerConfig.displaySelection[v[1]] or LoggerConfig.displayAll ) then
			table.insert(LoggerDisplay, i);
		end
	end
	if ( LoggerConfig.displayAll ) then
		LoggerFrameDisplayed:SetText(LOGGER_DISPLAYED..ALL);
	else
		LoggerFrameDisplayed:SetText(LOGGER_DISPLAYED..#LoggerDisplay);
	end
end

function LoggerFrame_Update()
	if ( LoggerSetDisplayPending ) then
		LoggerSetDisplay();
		LoggerSetDisplayPending = nil;
	end

	local offset = FauxScrollFrame_GetOffset(LoggerScrollFrame);
	local i, record;
	local type, dateTime, text, items;
	items = #LoggerDisplay;

	LoggerFrameTopArrow:Hide();
	LoggerFrameBottomArrow:Hide();

	for i = 1, 29 do
		if ( LoggerDisplay[i + offset] ) then
			record = LoggerDisplay[i + offset];
		else
			record = 0;
		end
		if ( LoggerLog[record] ) then

			if ( record == LoggerSelectionTop ) then
				LoggerFrameTopArrow:ClearAllPoints();
				LoggerFrameTopArrow:SetPoint( "TOPLEFT", _G["LoggerRecordDisplay"..i], "TOPLEFT", 0, 0 );
				LoggerFrameTopArrow:SetPoint( "BOTTOMLEFT", _G["LoggerRecordDisplay"..i], "BOTTOMLEFT", 0, 0 );
				LoggerFrameTopArrow:Show();
			elseif ( record == LoggerSelectionBottom ) then
				LoggerFrameBottomArrow:ClearAllPoints();
				LoggerFrameBottomArrow:SetPoint( "TOPLEFT", _G["LoggerRecordDisplay"..i], "TOPLEFT", 0, 0 );
				LoggerFrameBottomArrow:SetPoint( "BOTTOMLEFT", _G["LoggerRecordDisplay"..i], "BOTTOMLEFT", 0, 0 );
				LoggerFrameBottomArrow:Show();
			end
			if ( record >= LoggerSelectionTop and record <= LoggerSelectionBottom ) then
				_G["LoggerRecordDisplay"..i.."ButtonBackground"]:Show();
			else
				_G["LoggerRecordDisplay"..i.."ButtonBackground"]:Hide();
			end

			type, dateTime, text = LoggerReadRecord(record);
			_G["LoggerRecordDisplay"..i.."Text"]:SetTextColor(LoggerTypes[type].color.r, LoggerTypes[type].color.g, LoggerTypes[type].color.b);
			_G["LoggerRecordDisplay"..i.."DateTime"]:SetText( dateTime );
			_G["LoggerRecordDisplay"..i.."Text"]:SetText( text );
		else
			_G["LoggerRecordDisplay"..i.."DateTime"]:SetText( "" );
			_G["LoggerRecordDisplay"..i.."Text"]:SetText("");
		end

	end

	if ( items <= 29 ) then
		LoggerScrollFrame:Hide();
	else
		LoggerScrollFrame:Show();
	end

	LoggerFrameEntries:SetText(LOGGER_ENTRIES..#LoggerLog);

	FauxScrollFrame_Update(LoggerScrollFrame, items, 29, 12 );

	if ( LoggerScrollToBottom ) then
		local _, max = LoggerScrollFrameScrollBar:GetMinMaxValues();
		LoggerScrollFrameScrollBar:SetValue(max);
		LoggerScrollToBottom = nil;
	end

end

function LoggerRecordDisplayButton_OnClick(self)
	local record = FauxScrollFrame_GetOffset(LoggerScrollFrame);

	if ( LoggerDisplay[record + self:GetParent():GetID()] ) then
		record = LoggerDisplay[record + self:GetParent():GetID()];
	else
		record = 0
	end

	if ( record > #LoggerLog ) then
		return
	end

	if ( LoggerSelectionBottom == 0 and LoggerSelectionTop == 0 ) then
		LoggerSelectionTop = record;

	elseif ( LoggerSelectionTop == record ) then
		LoggerSelectionTop = 0;
		LoggerSelectionBottom = 0;

	elseif ( LoggerSelectionBottom == record ) then
		LoggerSelectionTop = 0;
		LoggerSelectionBottom = 0;

	elseif ( LoggerSelectionTop > 0 ) then
		if ( record > LoggerSelectionTop ) then
			LoggerSelectionBottom = record;
		else
			if ( LoggerSelectionBottom > 0 ) then
				LoggerSelectionTop = record;
			else
				LoggerSelectionBottom = LoggerSelectionTop;
				LoggerSelectionTop = record;
			end
		end
	end
	LoggerSummaryFrame:Hide();
	LoggerFrame_Update();
end

function Logger_SecondsToTime(seconds, noSeconds)
	local time = "";
	local tempTime;
	seconds = floor(seconds);
	if ( seconds >= 86400  ) then
		tempTime = floor(seconds / 86400);
		time = format(DAY_ONELETTER_ABBR, tempTime).."  ";
		seconds = mod(seconds, 86400);
	end
	if ( seconds >= 3600  ) then
		tempTime = floor(seconds / 3600);
		time = time..format(HOUR_ONELETTER_ABBR, tempTime).."  ";
		seconds = mod(seconds, 3600);
	end
	if ( seconds >= 60  ) then
		tempTime = floor(seconds / 60);
		time = time..format(MINUTE_ONELETTER_ABBR, tempTime).."  ";
		seconds = mod(seconds, 60);
	end
	if ( seconds > 0 and not noSeconds ) then
		seconds = format("%d", seconds);
		time = time..format(SECOND_ONELETTER_ABBR, seconds);
	end
	return time;
end

function LoggerSummaryButton_OnClick()
	if ( LoggerSummaryFrame:IsVisible() ) then
		LoggerSummaryFrame:Hide();
		return
	end

	local top, bottom;
	if ( LoggerSelectionTop == 0 or LoggerSelectionBottom == 0 ) then
		top = 1;
		bottom = #LoggerLog;
	else
		top = LoggerSelectionTop;
		bottom = LoggerSelectionBottom;
	end

	local text = LOGGER_TIMEFRAME;
	text = text..date("%c", LoggerLog[top][2]).." - \n"..date("%c", LoggerLog[bottom][2])..
	"\n("..Logger_SecondsToTime( LoggerLog[bottom][2] - LoggerLog[top][2] )..")\n\n";

	local mobs = {};
	local items = {};
	local faction = {};
	local deaths = 0;
	local kills = 0;
	local expGained = 0;
	local coins = 0;
	local coinEvents = 0;
	local i, item, num;

	for i = top, bottom do
		-- Player died
		if ( LoggerLog[i][1] == 2 ) then
			deaths = deaths + 1;

		-- mob killed
		elseif ( LoggerLog[i][1] == 3 ) then
			if ( not mobs[LoggerLog[i][3]] ) then
				mobs[LoggerLog[i][3]] = 1;
			else
				mobs[LoggerLog[i][3]] = mobs[LoggerLog[i][3]] + 1;
			end
			kills = kills + 1;

		-- coin loots
		elseif ( LoggerLog[i][1] == 14 ) then
			coinEvents = coinEvents + 1;
			coins = coins + LoggerLog[i][3] + LoggerLog[i][4] * 100 + LoggerLog[i][5] * 10000;
		
		-- item looted
		elseif ( LoggerLog[i][1] == 4 ) then

			if ( LoggerLog[i][6] ) then
				num = LoggerLog[i][6];
			else
				num = 1;
			end

			if ( not items[LoggerLog[i][5]] ) then
				items[LoggerLog[i][5]] = {};
			end
			if ( not items[LoggerLog[i][5]][LoggerLog[i][3]] ) then
				items[LoggerLog[i][5]][LoggerLog[i][3]] = num;
			else
				items[LoggerLog[i][5]][LoggerLog[i][3]] = items[LoggerLog[i][5]][LoggerLog[i][3]] + num;
			end

		-- faction change
		elseif ( LoggerLog[i][1] == 5 ) then
			if ( not faction[LoggerLog[i][4]] ) then
				faction[LoggerLog[i][4]] = LoggerLog[i][3];
			else
				faction[LoggerLog[i][4]] = faction[LoggerLog[i][4]] + LoggerLog[i][3];
			end

		-- exp
		elseif ( LoggerLog[i][1] == 10 ) then
			expGained = expGained + LoggerLog[i][3];
		end
	end

	local x, y;

	if ( deaths > 0 ) then
		text = text..DEATHS..": "..deaths.."\n\n";
	end

	if ( expGained > 0 ) then
		text = text..string.format(LOGGER_EXPERIENCE_GAIN_LOG, expGained).."\n\n";
	end

	for x, y in pairs(faction) do
		if ( y > 0 ) then
			text = text.."+";
		end
		text = text..y.." "..x.."\n";
	end
	if ( next(faction) ) then
		text = text.."\n";
	end

	if ( coins > 0 ) then
		text = text..LOGGER_COIN_EVENTS..coinEvents.."\n";
		text = text..LOGGER_COIN_LOOT..GetMoneyString(coins).."\n";
		text = text.."\n";
	end
	
	if ( kills > 0 ) then
		text = text..KILLS..": "..kills.."\n\n";

		for x, y in pairs(mobs) do
			text = text..x..LOGGER_KILLED..y.."\n";
		end
		text = text.."\n";
	end

	local colorNum, colorArr, color, item, loots;
	for colorNum, colorArr in pairs(items) do
		_, _, _, color = GetItemQualityColor(colorNum);
		for item, loots in pairs(colorArr) do
			text = text.."|c"..color..item.."|r"..LOGGER_LOOTED..loots.."\n";
		end
	end

--debugmsg(LoggerSummaryFrameScrollFrameScrollBar:GetValue());
	LoggerSummaryFrame:Show();
	LoggerSummaryFrameScrollFrameEditBox:SetText(text);
--	LoggerSummaryFrameScrollFrame:UpdateScrollChildRect();

--	ScrollingEdit_OnUpdate(LoggerSummaryFrameScrollFrame);
--	LoggerSummaryFrameScrollFrame:UpdateScrollChildRect();

--debugmsg(LoggerSummaryFrameScrollFrameScrollBar:GetValue());

	LoggerSummaryFrameScrollFrameScrollBar:SetValue(0);
--	LoggerSummaryFrameScrollFrame:UpdateScrollChildRect();

--debugmsg(LoggerSummaryFrameScrollFrameScrollBar:GetValue());
end

function LoggerClearEntries()
	LoggerLog = {};
	LoggerSelectionTop = 0;
	LoggerSelectionBottom = 0;
	local i;
	for i = 1, 29 do
		_G["LoggerRecordDisplay"..i.."ButtonBackground"]:Hide();
	end
	LoggerSummaryFrame:Hide();
	LoggerFrame_Update();
end

-- set the tooltip when mouseovering entries
function LoggerRecordDisplayButton_OnEnter(self)
	local offset = FauxScrollFrame_GetOffset(LoggerScrollFrame);
	local record, text, recordText, type;

	if ( LoggerDisplay[self:GetParent():GetID() + offset] ) then
		record = LoggerDisplay[self:GetParent():GetID() + offset];
	else
		record = 0;
	end
	if ( LoggerLog[record] ) then
		local dateStamp = time(date("*t"));
		type, _, recordText = LoggerReadRecord(record);

		text = Logger_SecondsToTime(dateStamp - LoggerLog[record][2], true);
		if ( text ~= "" ) then
			text = "Entry age: "..text;
		end

		-- add a \n after 100 characters
		if ( #recordText > 100 ) then
			local space = string.find(recordText, " ", 100, true);
			if ( space ) then
				recordText =	string.sub( recordText, 1, space )..
						"\n"..
						string.sub( recordText, space + 1 )
				;
			end
		end
		-- add a \n after 200 characters
		if ( #recordText > 200 ) then
			local space = string.find(recordText, " ", 200, true);
			if ( space ) then
				recordText =	string.sub( recordText, 1, space )..
						"\n"..
						string.sub( recordText, space + 1 )
				;
			end
		end

		GameTooltip_SetDefaultAnchor(GameTooltip, self)
		GameTooltip:SetText(recordText, LoggerTypes[type].color.r, LoggerTypes[type].color.g, LoggerTypes[type].color.b, 1);
		if ( text ~= "" ) then
			GameTooltip:AddLine(text, .5, .5, .5);
		end
		GameTooltip:Show();
	end
end

-- this changes the number of lines the arrow buttons on the scroll bar scroll to one line (blizzard's default was 10)
function LoggerSetScrollBarButtons()
	local up = function(self)
		local parent = self:GetParent();
		parent:SetValue(parent:GetValue() - 12);
		PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON);
	end
	local down = function(self)
		local parent = self:GetParent();
		parent:SetValue(parent:GetValue() + 12);
		PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON);
	end
	LoggerScrollFrameScrollBarScrollUpButton:SetScript("OnClick", up);
	LoggerScrollFrameScrollBarScrollDownButton:SetScript("OnClick", down);
end

function LoggerNextButton_OnClick()
	local target = LoggerSearchEditBox:GetText();
	local haystack, i;

	LoggerSearchErrorText:SetText("");

	if ( target == "" ) then
		return
	end

	local s = 1;
	local e = #LoggerDisplay;

	-- if selection
	if ( LoggerSelectionTop > 0 ) then

		-- if selection isn't the bottom most line
		if ( LoggerLog[LoggerSelectionTop + 1] ) then

			-- find the displayed line equal to or the first displayed line after current selection
			for i in ipairs(LoggerDisplay) do
				if ( LoggerDisplay[i] == LoggerSelectionTop ) then
					s = i + 1;
					break;
				elseif ( LoggerDisplay[i] > LoggerSelectionTop ) then
					s = i;
					break;
				end
			end
		end
	end

	target = string.lower(target);
	while ( true ) do
		for i = s, e do
			haystack = LoggerTypes[ LoggerLog[ LoggerDisplay[i] ][1] ].onRead(LoggerDisplay[i]);
			if ( string.find( string.lower(haystack), target, 1, true ) ) then
				LoggerSelectionBottom = 0;
				LoggerSelectionTop = LoggerDisplay[i];

				local newVal = (i * 12) - 112;		-- place selection in middle of screen
				local min, max = LoggerScrollFrameScrollBar:GetMinMaxValues();

				if ( newVal > max ) then
					newMax = max;
				end
				if ( newVal < min ) then
					newVal = min;
				end
				LoggerScrollFrameScrollBar:SetValue(newVal);
				LoggerFrame_Update();
				return
			end
		end

		-- text not found, start searching from top
		if ( s > 1 ) then
			e = s - 1;
			s = 1;
		else
			LoggerSearchErrorText:SetText(string.format(ERR_GUILD_PLAYER_NOT_FOUND_S, target));
			return
		end
	end

end