--[[
	enable = true,
	maxRecords = 1000,
	maxTime = 0,
	loaded = true,
	deaths = true,
	kills = true,
	loot = true,
	faction = true,
	zoneChange = true,
]]

function Logger_OptionsCheckButtonOnClick(self)
	local id = self:GetID();

	if ( not LoggerCheckButtons[id] ) then
		return
	end

	local type = LoggerCheckButtons[id].type;

	if ( self:GetChecked() ) then
		LoggerConfig[LoggerTypes[type].var] = true;
	else
		LoggerConfig[LoggerTypes[type].var] = false;
	end
end

function Logger_OptionsCheckButtonOnLoad(self)
	local id = self:GetID();

	if ( LoggerCheckButtons[id] ) then
		if ( not LoggerCheckButtons[id].displayOnly ) then
			self:Show();
			_G[self:GetName().."Text"]:SetText(LoggerCheckButtons[id].text);
		end
	end

	if ( id > 1 ) then
		if ( id == 12 ) then
			self:SetPoint("TOPLEFT", LoggerOptionsFrame, "TOPLEFT", 225, -198);
		elseif ( LoggerCheckButtons[id] and LoggerCheckButtons[id - 1].displayOnly ) then
			self:SetPoint("TOP", _G["LoggerEventCheckButton"..(id - 2)], "BOTTOM", 0, -3);
		else
			self:SetPoint("TOP", _G["LoggerEventCheckButton"..(id - 1)], "BOTTOM", 0, -3);
		end
	end
end

function Logger_OptionsCheckButtonOnShow(self)
	local id = self:GetID();
	if ( not LoggerCheckButtons[id] ) then
		return
	end
	local type = LoggerCheckButtons[id].type;

	if ( LoggerConfig[LoggerTypes[type].var] ) then
		self:SetChecked(true);
	else
		self:SetChecked(false);
	end
end

function LoggerSetMaxTime()
	local days = tonumber( LoggerMaxDaysEditBox:GetText() );
	local hours = tonumber( LoggerMaxHoursEditBox:GetText() );
	local minutes = tonumber( LoggerMaxMinutesEditBox:GetText() );
	local timeInSeconds = 0;

	if ( days ) then
		timeInSeconds = timeInSeconds + (days * 86400);
	end
	if ( hours ) then
		timeInSeconds = timeInSeconds + (hours * 3600);
	end
	if ( minutes ) then
		timeInSeconds = timeInSeconds + (minutes * 60);
	end

	LoggerConfig.maxTime = timeInSeconds;
end

function LoggerGetMaxTime()
	local seconds = LoggerConfig.maxTime;
	local tempTime;

	if ( seconds >= 86400 ) then
		tempTime = floor(seconds / 86400);
		LoggerMaxDaysEditBox:SetText(tempTime);
		seconds = mod(seconds, 86400);
	else
		LoggerMaxDaysEditBox:SetText(0);
	end
	if ( seconds >= 3600 ) then
		tempTime = floor(seconds / 3600);
		LoggerMaxHoursEditBox:SetText(tempTime);
		seconds = mod(seconds, 3600);
	else
		LoggerMaxHoursEditBox:SetText(0);
	end
	if ( seconds >= 60 ) then
		tempTime = floor(seconds / 60);
		LoggerMaxMinutesEditBox:SetText(tempTime);
		seconds = mod(seconds, 60);
	else
		LoggerMaxMinutesEditBox:SetText(0);
	end
end

function LoggerDisplaySelectionFrame_OnLoad(self)
	self:SetBackdropColor(
		TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
		TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
		TOOLTIP_DEFAULT_BACKGROUND_COLOR.b
	);

end

function Logger_DisplayCheckButtonOnClick(self)
	local id = self:GetID();

	if ( not LoggerCheckButtons[id] ) then
		return
	end

	local type = LoggerCheckButtons[id].type;

	if ( not LoggerConfig.displaySelection ) then
		LoggerConfig.displaySelection = {};
	end
	if ( self:GetChecked() ) then
		LoggerConfig.displaySelection[type] = true;
	else
		LoggerConfig.displaySelection[type] = false;
	end
end

function Logger_DisplayCheckButtonOnLoad(self)
	local id = self:GetID();

	if ( LoggerCheckButtons[id] ) then
		self:Show();
		_G[self:GetName().."Text"]:SetText(LoggerCheckButtons[id].text);
	end
	if ( id == 11 ) then
		self:SetPoint("TOPLEFT", LoggerDisplaySelectionFrame, "TOPLEFT", 250, -75);
	else
		self:SetPoint("TOP", _G["LoggerDisplayCheckButton"..(id - 1)], "BOTTOM", 0, -3);
	end
end

function Logger_DisplayCheckButtonOnShow(self)
	local id = self:GetID();
	if ( not LoggerCheckButtons[id] ) then
		return
	end
	local type = LoggerCheckButtons[id].type;

	if ( not LoggerConfig.displaySelection ) then
		LoggerConfig.displaySelection = {};
	end
	if ( LoggerConfig.displaySelection[type] ) then
		self:SetChecked(true);
	else
		self:SetChecked(false);
	end
end

function LoggerDisplayFrameDisableCheckButtons(toggle)
	local i = 1;
	if ( toggle ) then
		while ( _G["LoggerDisplayCheckButton"..i] ) do
			_G["LoggerDisplayCheckButton"..i]:Disable();
			_G["LoggerDisplayCheckButton"..i.."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			LoggerDisplayedEntriesLabel:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			i = i + 1;
		end
	else
		while ( _G["LoggerDisplayCheckButton"..i] ) do
			_G["LoggerDisplayCheckButton"..i]:Enable();
			_G["LoggerDisplayCheckButton"..i.."Text"]:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			LoggerDisplayedEntriesLabel:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			i = i + 1;
		end
	end
end

function LoggerShowAllCheckButton_OnClick(self)
	if (self:GetChecked()) then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF);
		LoggerConfig.displayAll = true;
		LoggerDisplayFrameDisableCheckButtons(true);
	else
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
		LoggerConfig.displayAll = false;
		LoggerDisplayFrameDisableCheckButtons(false);
	end
end